/* 

                          Firewall Builder

                 Copyright (C) 2003 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@fwbuilder.org

  $Id: ObjectListView.cpp,v 1.12 2006/10/22 04:39:36 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"
#include "global.h"

#include "fwbuilder/FWObject.h"
#include "fwbuilder/Resources.h"
#include "fwbuilder/FWObjectDatabase.h"

#include "FWWindow.h"
#include "ObjectListView.h"
#include "ObjectListViewItem.h"
#include "FWObjectDrag.h"


#include <qdragobject.h>
#include <qlistview.h>
#include <qpixmapcache.h>

#include <iostream>

using namespace std;
using namespace libfwbuilder;

/****************************************************************************
 *
 *    class ObjectListView
 *
 ****************************************************************************/

ObjectListView::ObjectListView(QWidget* parent, const char * name, WFlags f) :
    QListView(parent,name,f)
{
    setColumnWidthMode(0, QListView::Maximum);
    setColumnWidthMode(1, QListView::Maximum);
    setItemMargin( 2 );
    setFocusPolicy( QWidget::StrongFocus  );
    setFocus();
}

QDragObject* ObjectListView::dragObject()
{
    QListViewItem *ovi = currentItem();
    ObjectListViewItem *otvi=dynamic_cast<ObjectListViewItem*>(ovi);
    assert(otvi!=NULL);

    FWObject *obj = otvi->getFWObject();
    QString icn =
        Resources::global_res->getObjResourceStr(obj, "icon-ref").c_str();

    list<FWObject*> dragobj;
    dragobj.push_back(obj);

    FWObjectDrag    *drag = new FWObjectDrag(dragobj, this);
    //QPixmap          pm   = QPixmap::fromMimeSource( icn_filename );

    QPixmap pm;
    if ( ! QPixmapCache::find( icn, pm) )
    {
        pm = QPixmap::fromMimeSource( icn );
        QPixmapCache::insert( icn, pm);
    }
    
    drag->setPixmap( pm,
                     QPoint( pm.rect().width() / 2,
                             pm.rect().height() / 2 ) );

    return drag;
}

void ObjectListView::dragEnterEvent( QDragEnterEvent *ev)
{
    ev->accept( FWObjectDrag::canDecode(ev) );
}

void ObjectListView::dropEvent(QDropEvent *ev)
{
    emit dropped(ev);
}

void ObjectListView::keyPressEvent( QKeyEvent* ev )
{
    qDebug("ObjectListView::keyPressEvent( QKeyEvent* ev )");
    if (ev->key()==Qt::Key_Delete)
    {
        emit delObject_sign();
    }
    QListView::keyPressEvent(ev);
}



    
